﻿// 
//  (c) Microsoft Corporation. See LICENSE.TXT file for licensing details
//  
namespace BackendWebRole
{
    using System;
    using System.Collections.Generic;
    using System.Configuration;
    using Microsoft.ServiceBus;
    using Microsoft.ServiceBus.Messaging;

    public class DeviceCommandSender
    {
        static readonly string ManagementKey;
        static readonly string NamespaceName;
        readonly MessagingFactory messagingFactory;
        readonly Uri namespaceUri;
        readonly TopicClient topicClient;

        static DeviceCommandSender()
        {
            NamespaceName = ConfigurationManager.AppSettings["serviceBusNamespace"];
            ManagementKey = ConfigurationManager.AppSettings["managementKey"];
        }

        public DeviceCommandSender()
        {
            this.namespaceUri = ServiceBusEnvironment.CreateServiceUri("sb", NamespaceName, String.Empty);
            this.messagingFactory = MessagingFactory.Create(this.namespaceUri, TokenProvider.CreateSharedSecretTokenProvider("owner", ManagementKey));
            this.topicClient = this.messagingFactory.CreateTopicClient("devices");
        }

        public void Broadcast(IDictionary<string, object> data)
        {
            var bm = new BrokeredMessage();
            foreach (var keyValuePair in data)
            {
                bm.Properties.Add(keyValuePair);
            }
            bm.Properties.Add("Broadcast", true);
            this.topicClient.Send(bm);
        }

        public void SendToDevice(string deviceId, IDictionary<string, object> data)
        {
            var bm = new BrokeredMessage();
            foreach (var keyValuePair in data)
            {
                bm.Properties.Add(keyValuePair);
            }
            bm.Properties.Add("DeviceId", deviceId);
            this.topicClient.Send(bm);
        }
    }
}