﻿// 
//  (c) Microsoft Corporation. See LICENSE.TXT file for licensing details
//  
namespace BackendWebRole
{
    using System;
    using System.Configuration;
    using Microsoft.ServiceBus;
    using Microsoft.ServiceBus.Messaging;

    public class DeviceEventReceiver
    {
        static readonly string ManagementKey;
        static readonly string NamespaceName;
        readonly MessagingFactory messagingFactory;
        readonly Uri namespaceUri;
        readonly SubscriptionClient subscriptionClient;

        static DeviceEventReceiver()
        {
            NamespaceName = ConfigurationManager.AppSettings["serviceBusNamespace"];
            ManagementKey = ConfigurationManager.AppSettings["managementKey"];
        }

        public DeviceEventReceiver()
        {
            this.namespaceUri = ServiceBusEnvironment.CreateServiceUri("sb", NamespaceName, String.Empty);
            this.messagingFactory = MessagingFactory.Create(this.namespaceUri, TokenProvider.CreateSharedSecretTokenProvider("owner", ManagementKey));
            this.subscriptionClient = this.messagingFactory.CreateSubscriptionClient("events", "main");
        }

        public void Start()
        {
            this.subscriptionClient.BeginReceive(this.DoneReceiving, null);
        }

        void DoneReceiving(IAsyncResult ar)
        {
            try
            {
                var msg = this.subscriptionClient.EndReceive(ar);
                if (msg != null)
                {
                    msg.Complete();
                }
                else
                {
                    return;
                }
            }
            catch (Exception)
            {
                if (this.subscriptionClient.IsClosed)
                {
                    return;
                }
            }
            finally
            {
                this.subscriptionClient.BeginReceive(this.DoneReceiving, null);
            }
        }

        public void Stop()
        {
            this.subscriptionClient.Close();
        }
    }
}