﻿// 
//  (c) Microsoft Corporation. See LICENSE.TXT file for licensing details
//  
namespace ServiceBusMicroTest
{
    using System;
    using System.IO;
    using System.Net;
    using Microsoft.ServiceBus.Micro;
    using Microsoft.SPOT;
    using ServiceBusApp;

    public class Program
    {
        public static void Main()
        {
            var settings = new ApplicationSettings();
            PerformProvisioning(settings, "0123456");

            var tokenProvider = new TokenProvider(settings.DeviceAccount, settings.DeviceKey);
            var sendClient = new MessagingClient(settings.EventSubmissionUri, tokenProvider);
            sendClient.Send(new SimpleMessage {Properties = {{"myheader", "myvalue"}}});

            var receiveClient = new MessagingClient(settings.DeviceSubscriptionUri, tokenProvider);
            SimpleMessage msg;
            do
            {
                msg = receiveClient.Receive(TimeSpan.FromTicks(TimeSpan.TicksPerSecond*50), ReceiveMode.PeekLock);
            }
            while (msg == null);
            Debug.Print(msg.Properties["Temperature"].ToString());
            msg.Complete();
        }

        static bool PerformProvisioning(ApplicationSettings settings, string deviceId)
        {
            try
            {
                var serverAddress = "http://127.0.0.1:81";
                var wr = WebRequest.Create(serverAddress + "/Provisioning.svc/setup");
                wr.Method = "POST";
                wr.ContentLength = 0;
                wr.Headers.Add("P-DeviceId", deviceId);

                using (var wq = (HttpWebResponse) wr.GetResponse())
                {
                    if (wq.StatusCode ==
                        HttpStatusCode.OK)
                    {
                        settings.DeviceAccount = wq.Headers["P-DeviceAccount"];
                        settings.DeviceKey = wq.Headers["P-DeviceKey"];

                        var subUriString = wq.Headers["P-DeviceSubscriptionUri"];
                        if (subUriString != null &&
                            subUriString.Length > 0)
                        {
                            settings.DeviceSubscriptionUri = subUriString[0] == '/' ? new Uri(serverAddress + subUriString) : new Uri(subUriString);
                        }
                        var eventUriString = wq.Headers["P-EventSubmissionUri"];
                        if (eventUriString != null &&
                            eventUriString.Length > 0)
                        {
                            settings.EventSubmissionUri = eventUriString[0] == '/' ? new Uri(serverAddress + eventUriString) : new Uri(eventUriString);
                        }
                        settings.NetworkProvisioningCompleted = true;
                        return true;
                    }
                    else
                    {
                        Debug.Print(new StreamReader(wq.GetResponseStream()).ReadToEnd());
                    }
                }
            }
            catch (Exception e)
            {
                return false;
            }
            return false;
        }
    }
}